@@ -35,7 +35,7 @@
  * I decided to write a program to do it.
  *
  * At first I just wrote a really ugly hack to do it, which I then planned
- * on using once just to see how it worked, and then posting a URL and 
+ * on using once just to see how it worked, and then posting a URL and
  * laughing about it on #gimp.  As it turns out, tigert thought it actually
  * had potential to be a useful plugin, so I started adding features and
  * and making a nice UI.
@@ -50,6 +50,7 @@
 
 #include "config.h"
 
+#include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -61,8 +62,6 @@
 
 #include "libgimp/stdplugins-intl.h"
 
-#include "pixmaps/eek.xpm"
-
 
 /* Typedefs */
 
@@ -81,18 +80,8 @@ typedef struct
   gint  cellspacing;
 } GTMValues;
 
-typedef struct
-{
-  gint run;
-} GTMInterface;
-
 /* Variables */
 
-static GTMInterface bint =
-{
-  FALSE  /* run */
-};
-
 static GTMValues gtmvals =
 {
   "Made with GIMP Table Magic",  /* caption text */
@@ -111,30 +100,29 @@ static GTMValues gtmvals =
 /* Declare some local functions */
 
 static void   query      (void);
-static void   run        (gchar   *name,
-                          gint     nparams,
-                          GParam  *param,
-                          gint    *nreturn_vals,
-                          GParam **return_vals);
+static void   run        (const gchar      *name,
+                          gint              nparams,
+                          const GimpParam  *param,
+                          gint             *nreturn_vals,
+                          GimpParam       **return_vals);
 
-static gint   save_image  (gchar     *filename,
-			   GDrawable *drawable);
-static gint   save_dialog (gint32     image_ID);
+static gboolean save_image          (const gchar  *filename,
+                                     GimpDrawable *drawable);
+static gboolean save_dialog         (gint32        image_ID);
 
-static gint   color_comp               (guchar    *buffer,
-					guchar    *buf2);
-static void   save_ok_callback         (GtkWidget *widget,
-					gpointer   data);
+static gboolean color_comp             (guchar    *buffer,
+                                        guchar    *buf2);
 static void   gtm_caption_callback     (GtkWidget *widget,
-					gpointer   data);
+                                        gpointer   data);
 static void   gtm_cellcontent_callback (GtkWidget *widget,
-					gpointer   data);
+                                        gpointer   data);
 static void   gtm_clwidth_callback     (GtkWidget *widget,
-					gpointer   data);
+                                        gpointer   data);
 static void   gtm_clheight_callback    (GtkWidget *widget,
-					gpointer   data);
+                                        gpointer   data);
 
-GPlugInInfo PLUG_IN_INFO =
+
+GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
@@ -148,210 +136,252 @@ MAIN ()
 static void
 query (void)
 {
-  static GParamDef save_args[] =
+  static GimpParamDef save_args[] =
   {
-    { PARAM_INT32, "run_mode", "Interactive" },
-    { PARAM_IMAGE, "image", "Input image" },
-    { PARAM_DRAWABLE, "drawable", "Drawable to save" },
-    { PARAM_STRING, "filename", "The name of the file to save the image in" },
-    { PARAM_STRING, "raw_filename", "The name of the file to save the image in" }
+    { GIMP_PDB_INT32,    "run_mode",     "Interactive" },
+    { GIMP_PDB_IMAGE,    "image",        "Input image" },
+    { GIMP_PDB_DRAWABLE, "drawable",     "Drawable to save" },
+    { GIMP_PDB_STRING,   "filename",     "The name of the file to save the image in" },
+    { GIMP_PDB_STRING,   "raw_filename", "The name of the file to save the image in" }
   };
-  static gint nsave_args = sizeof (save_args) / sizeof (save_args[0]);
 
-  gimp_install_procedure ("file_GTM_save",
+  gimp_install_procedure ("file_gtm_save",
                           "GIMP Table Magic",
                           "Allows you to draw an HTML table in GIMP. See help for more info.",
                           "Daniel Dunbar",
                           "Daniel Dunbar",
                           "1998",
-                          "<Save>/HTML",
+                          _("HTML table"),
 			  "RGB*, GRAY*",
-                          PROC_PLUG_IN,
-                          nsave_args, 0,
+                          GIMP_PLUGIN,
+                          G_N_ELEMENTS (save_args), 0,
                           save_args, NULL);
 
-  gimp_register_save_handler ("file_GTM_save",
-			      "html,htm",
-			      "");
+  gimp_register_file_handler_mime ("file_gtm_save", "text/html");
+  gimp_register_save_handler ("file_gtm_save", "html,htm", "");
 }
 
 static void
-run (gchar   *name,
-     gint     nparams,
-     GParam  *param,
-     gint    *nreturn_vals,
-     GParam **return_vals)
+run (const gchar      *name,
+     gint              nparams,
+     const GimpParam  *param,
+     gint             *nreturn_vals,
+     GimpParam       **return_vals)
 {
-  static GParam values[2];
-  GStatusType   status = STATUS_SUCCESS;
-  GDrawable *drawable;
+  static GimpParam   values[2];
+  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
+  GimpDrawable      *drawable;
 
-  INIT_I18N_UI();
+  INIT_I18N ();
 
   drawable = gimp_drawable_get (param[2].data.d_int32);
 
   *nreturn_vals = 1;
   *return_vals  = values;
-  values[0].type          = PARAM_STATUS;
-  values[0].data.d_status = STATUS_EXECUTION_ERROR;
 
-  gimp_get_data ("file_GTM_save", &gtmvals);
+  values[0].type          = GIMP_PDB_STATUS;
+  values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
+
+  gimp_get_data ("file_gtm_save", &gtmvals);
 
   if (save_dialog (param[1].data.d_int32))
     {
       if (save_image (param[3].data.d_string, drawable))
 	{
-	  gimp_set_data ("file_GTM_save", &gtmvals, sizeof (GTMValues));
+	  gimp_set_data ("file_gtm_save", &gtmvals, sizeof (GTMValues));
 	}
       else
 	{
-	  status = STATUS_EXECUTION_ERROR;
+	  status = GIMP_PDB_EXECUTION_ERROR;
 	}
     }
   else
     {
-      status = STATUS_CANCEL;
+      status = GIMP_PDB_CANCEL;
     }
 
   values[0].data.d_status = status;
 }
 
-static gint
-save_image (gchar     *filename,
-	    GDrawable *drawable)
+static gboolean
+save_image (const gchar  *filename,
+	    GimpDrawable *drawable)
 {
-  int row,col, cols, rows, x, y;
-  int colcount, colspan, rowspan;
-  /* This works only in gcc - not allowed according */
-  /* to ANSI C */
-  /*int palloc[drawable->width][drawable->height];*/
-  int *palloc;
-  guchar *buffer, *buf2;
-  gchar *width, *height;
-  GPixelRgn pixel_rgn;
-  char *name;
+  gint          row,col, cols, rows, x, y;
+  gint          colcount, colspan, rowspan;
+  gint         *palloc;
+  guchar       *buffer, *buf2;
+  gchar        *width, *height;
+  GimpPixelRgn  pixel_rgn;
+  gchar        *name;
+  FILE         *fp;
 
-  FILE *fp, *fopen();
+  palloc = g_new (int, drawable->width * drawable->height);
 
-  palloc = malloc(drawable->width * drawable->height * sizeof(int));
+  fp = fopen (filename, "w");
+
+  if (! fp)
+    {
+      g_message (_("Could not open '%s' for writing: %s"),
+                 gimp_filename_to_utf8 (filename), g_strerror (errno));
+      return FALSE;
+    }
+
+  if (gtmvals.fulldoc)
+    {
+      fprintf (fp, "<HTML>\n<HEAD><TITLE>%s</TITLE></HEAD>\n<BODY>\n",
+               filename);
+      fprintf (fp, "<H1>%s</H1>\n",
+               filename);
+    }
+
+  fprintf (fp, "<TABLE BORDER=%d CELLPADDING=%d CELLSPACING=%d>\n",
+           gtmvals.border, gtmvals.cellpadding, gtmvals.cellspacing);
 
-  fp = fopen(filename, "w");
-  if (gtmvals.fulldoc) {
-    fprintf (fp,"<HTML>\n<HEAD><TITLE>%s</TITLE></HEAD>\n<BODY>\n",filename);
-    fprintf (fp,"<H1>%s</H1>\n",filename);
-  }
-  fprintf (fp,"<TABLE BORDER=%d CELLPADDING=%d CELLSPACING=%d>\n",gtmvals.border,gtmvals.cellpadding,gtmvals.cellspacing);
   if (gtmvals.caption)
-    fprintf (fp,"<CAPTION>%s</CAPTION>\n",gtmvals.captiontxt); 
+    fprintf (fp, "<CAPTION>%s</CAPTION>\n",
+             gtmvals.captiontxt);
 
-  name = g_strdup_printf (_("Saving %s:"), filename);
+  name = g_strdup_printf (_("Saving '%s'..."),
+                          gimp_filename_to_utf8 (filename));
   gimp_progress_init (name);
   g_free (name);
 
-  gimp_pixel_rgn_init (&pixel_rgn, drawable, 0, 0, drawable->width, drawable->height, FALSE, FALSE);
+  gimp_pixel_rgn_init (&pixel_rgn, drawable,
+                       0, 0, drawable->width, drawable->height,
+                       FALSE, FALSE);
 
-  cols = drawable->width;
-  rows = drawable->height;
-  buffer = g_new(guchar,drawable->bpp);
-  buf2 = g_new(guchar,drawable->bpp);
+  cols   = drawable->width;
+  rows   = drawable->height;
+  buffer = g_new (guchar, drawable->bpp);
+  buf2   = g_new (guchar, drawable->bpp);
+
+  width  = g_malloc (2);
+  height = g_malloc (2);
+  sprintf (width, " ");
+  sprintf (height, " ");
+
+  if (strcmp (gtmvals.clwidth, "") != 0)
+    {
+      width = g_malloc (strlen (gtmvals.clwidth) + 11);
+      sprintf (width," WIDTH=\"%s\"", gtmvals.clwidth);
+    }
+
+  if (strcmp (gtmvals.clheight, "") != 0)
+    {
+      height = g_malloc (strlen (gtmvals.clheight) + 13);
+      sprintf (height, " HEIGHT=\"%s\" ", gtmvals.clheight);
+    }
 
-  width = malloc (2);
-  height = malloc (2);
-  sprintf(width," ");
-  sprintf(height," ");
-  if (strcmp (gtmvals.clwidth, "") != 0) {
-    width = malloc (strlen (gtmvals.clwidth) + 11);
-    sprintf(width," WIDTH=\"%s\"",gtmvals.clwidth);
-  }
-  if (strcmp (gtmvals.clheight, "") != 0) {
-    height = malloc (strlen (gtmvals.clheight) + 13);
-    sprintf(height," HEIGHT=\"%s\" ",gtmvals.clheight);
-  }  
-  
   /* Initialize array to hold ROWSPAN and COLSPAN cell allocation table */
 
-  for (row=0; row < rows; row++)
-    for (col=0; col < cols; col++)
-      palloc[drawable->width * row + col]=1;
+  for (row = 0; row < rows; row++)
+    for (col = 0; col < cols; col++)
+      palloc[drawable->width * row + col] = 1;
 
-  colspan=0;
-  rowspan=0;
+  colspan = 0;
+  rowspan = 0;
 
-  for (y = 0; y < rows; y++) {
-    fprintf (fp,"   <TR>\n");
-    for (x = 0; x < cols; x++) {
-      gimp_pixel_rgn_get_pixel(&pixel_rgn, buffer, x, y);
+  for (y = 0; y < rows; y++)
+    {
+      fprintf (fp,"   <TR>\n");
 
-      /* Determine ROWSPAN and COLSPAN */
+      for (x = 0; x < cols; x++)
+        {
+          gimp_pixel_rgn_get_pixel (&pixel_rgn, buffer, x, y);
 
-      if (gtmvals.spantags) { 
-	col=x;
-	row=y;
-	colcount=0;
-	colspan=0;
-	rowspan=0;
-	gimp_pixel_rgn_get_pixel(&pixel_rgn, buf2, col, row);
-	
-	while (color_comp(buffer,buf2) && palloc[drawable->width * row + col] == 1 && row < drawable->height) {
-	  while (color_comp(buffer,buf2) && palloc[drawable->width * row + col] == 1 && col < drawable->width ) {
-	    colcount++;
-	    col++;
-	    gimp_pixel_rgn_get_pixel(&pixel_rgn, buf2, col, row);
-	  }
-	  
-	  if (colcount != 0) {
-	    row++;
-	    rowspan++;
-	  }
-	  
-	  if (colcount < colspan || colspan == 0)
-	    colspan=colcount;
-	  
-	  col=x;
-	  colcount=0;
-	  gimp_pixel_rgn_get_pixel(&pixel_rgn, buf2, col, row);
-	}
-	
-	if (colspan > 1 || rowspan > 1) {
-	  for (row=0; row < rowspan; row++)
-	    for (col=0; col < colspan; col++)
-	      palloc[drawable->width * (row+y) + (col+x)]=0;
-	  palloc[drawable->width * y + x]=2;
-	}
-      }
+          /* Determine ROWSPAN and COLSPAN */
 
-      if (palloc[drawable->width * y + x]==1)
-	fprintf (fp,"      <TD%s%sBGCOLOR=#%02x%02x%02x>",width,height,buffer[0],buffer[1],buffer[2]);
+          if (gtmvals.spantags)
+            {
+              col      = x;
+              row      = y;
+              colcount = 0;
+              colspan  = 0;
+              rowspan  = 0;
 
-      if (palloc[drawable->width * y + x]==2)
-	fprintf (fp,"      <TD ROWSPAN=\"%d\" COLSPAN=\"%d\"%s%sBGCOLOR=#%02x%02x%02x>",rowspan,colspan,width,height,buffer[0],buffer[1],buffer[2]);
+              gimp_pixel_rgn_get_pixel (&pixel_rgn, buf2, col, row);
 
-      if (palloc[drawable->width * y + x]!=0) {
-	if (gtmvals.tdcomp)
-	  fprintf (fp,"%s</TD>\n",gtmvals.cellcontent);
-	else 
-	  fprintf (fp,"\n      %s\n      </TD>\n",gtmvals.cellcontent);
-      }
+              while (color_comp (buffer,buf2) &&
+                     palloc[drawable->width * row + col] == 1 &&
+                     row < drawable->height)
+                {
+                  while (color_comp (buffer,buf2) &&
+                         palloc[drawable->width * row + col] == 1 &&
+                         col < drawable->width )
+                    {
+                      colcount++;
+                      col++;
+
+                      gimp_pixel_rgn_get_pixel (&pixel_rgn, buf2, col, row);
+                    }
+
+                  if (colcount != 0)
+                    {
+                      row++;
+                      rowspan++;
+                    }
+
+                  if (colcount < colspan || colspan == 0)
+                    colspan = colcount;
+
+                  col = x;
+                  colcount = 0;
+
+                  gimp_pixel_rgn_get_pixel (&pixel_rgn, buf2, col, row);
+                }
+
+              if (colspan > 1 || rowspan > 1)
+                {
+                  for (row = 0; row < rowspan; row++)
+                    for (col = 0; col < colspan; col++)
+                      palloc[drawable->width * (row + y) + (col + x)] = 0;
+
+                  palloc[drawable->width * y + x] = 2;
+                }
+            }
+
+          if (palloc[drawable->width * y + x] == 1)
+            fprintf (fp, "      <TD%s%sBGCOLOR=#%02x%02x%02x>",
+                     width, height, buffer[0], buffer[1], buffer[2]);
+
+          if (palloc[drawable->width * y + x] == 2)
+            fprintf (fp,"      <TD ROWSPAN=\"%d\" COLSPAN=\"%d\"%s%sBGCOLOR=#%02x%02x%02x>",
+                     rowspan, colspan, width, height,
+                     buffer[0], buffer[1], buffer[2]);
+
+          if (palloc[drawable->width * y + x] != 0)
+            {
+              if (gtmvals.tdcomp)
+                fprintf (fp, "%s</TD>\n", gtmvals.cellcontent);
+              else
+                fprintf (fp, "\n      %s\n      </TD>\n", gtmvals.cellcontent);
+            }
+        }
+
+      fprintf (fp,"   </TR>\n");
+
+      gimp_progress_update ((double) y / (double) rows);
     }
-    fprintf (fp,"   </TR>\n");
-    gimp_progress_update ((double) y / (double) rows);
-  }
 
   if (gtmvals.fulldoc)
-    fprintf (fp,"</TABLE></BODY></HTML>\n");  
-  else fprintf (fp,"</TABLE>\n");
-  fclose(fp);
+    fprintf (fp, "</TABLE></BODY></HTML>\n");
+  else
+    fprintf (fp, "</TABLE>\n");
+
+  fclose (fp);
   gimp_drawable_detach (drawable);
-  free(width);
-  free(height);
 
-  free(palloc);
+  g_free (width);
+  g_free (height);
 
-  return 1;
+  g_free (palloc);
+
+  return TRUE;
 }
 
 static gint
-save_dialog (image_ID)
+save_dialog (gint32 image_ID)
 {
   GtkWidget *dlg;
   GtkWidget *main_vbox;
@@ -362,32 +392,21 @@ save_dialog (image_ID)
   GtkObject *adj;
   GtkWidget *entry;
   GtkWidget *toggle;
-
-  bint.run = FALSE;
+  gboolean   run;
 
   gimp_ui_init ("gtm", FALSE);
 
   dlg = gimp_dialog_new (_("GIMP Table Magic"), "gtm",
-			 gimp_standard_help_func, "filters/gtm.html",
-			 GTK_WIN_POS_MOUSE,
-			 FALSE, TRUE, FALSE,
+                         NULL, 0,
+			 gimp_standard_help_func, "file-gtm-save",
 
-			 _("OK"), save_ok_callback,
-			 NULL, NULL, NULL, TRUE, FALSE,
-			 _("Cancel"), gtk_widget_destroy,
-			 NULL, 1, NULL, FALSE, TRUE,
+			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
 			 NULL);
 
-  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
-                      GTK_SIGNAL_FUNC (gtk_main_quit),
-                      NULL);
-
-  /* Initialize Tooltips */
-  gimp_help_init ();
-
-  main_vbox = gtk_vbox_new (FALSE, 4);
-  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 6);
+  main_vbox = gtk_vbox_new (FALSE, 12);
+  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), main_vbox,
 		      TRUE, TRUE, 0);
 
@@ -397,19 +416,18 @@ save_dialog (image_ID)
       GtkWidget *label;
       GtkWidget *hbox;
 
-      frame = gtk_frame_new (_("Warning"));
+      frame = gimp_frame_new (_("Warning"));
       gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
       gtk_box_pack_start (GTK_BOX (main_vbox), frame, FALSE, FALSE, 0);
 
-      hbox = gtk_hbox_new (FALSE, 4);
-      gtk_container_set_border_width (GTK_CONTAINER (hbox), 4);
+      hbox = gtk_hbox_new (FALSE, 12);
       gtk_container_add (GTK_CONTAINER (frame), hbox);
-      
-      eek = gimp_pixmap_new (eek_xpm);
-      gtk_box_pack_start (GTK_BOX (hbox), eek, FALSE, FALSE, 4);
 
-      label = gtk_label_new (_("Are you crazy?\n\n"
-			       "You are about to create a huge\n"
+      eek = gtk_image_new_from_stock (GIMP_STOCK_WILBER_EEK,
+                                      GTK_ICON_SIZE_DIALOG);
+      gtk_box_pack_start (GTK_BOX (hbox), eek, FALSE, FALSE, 0);
+
+      label = gtk_label_new (_("You are about to create a huge\n"
 			       "HTML file which will most likely\n"
 			       "crash your browser."));
       gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
@@ -418,62 +436,60 @@ save_dialog (image_ID)
     }
 
   /* HTML Page Options */
-  frame = gtk_frame_new (_("HTML Page Options"));
-  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+  frame = gimp_frame_new (_("HTML Page Options"));
   gtk_box_pack_start (GTK_BOX (main_vbox), frame, FALSE, FALSE, 0);
 
-  vbox = gtk_vbox_new (FALSE, 2);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
+  vbox = gtk_vbox_new (FALSE, 6);
   gtk_container_add (GTK_CONTAINER (frame), vbox);
   gtk_widget_show (vbox);
 
-  toggle = gtk_check_button_new_with_label (_("Generate Full HTML Document"));
+  toggle = gtk_check_button_new_with_mnemonic (_("_Generate full HTML document"));
   gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);
-  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &gtmvals.fulldoc);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), gtmvals.fulldoc);
   gtk_widget_show (toggle);
+
   gimp_help_set_help_data (toggle,
 			   _("If checked GTM will output a full HTML document "
 			     "with <HTML>, <BODY>, etc. tags instead of just "
 			     "the table html."),
 			   NULL);
 
+  g_signal_connect (toggle, "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &gtmvals.fulldoc);
+
   gtk_widget_show (main_vbox);
   gtk_widget_show (frame);
 
   /* HTML Table Creation Options */
-  frame = gtk_frame_new (_("Table Creation Options"));
-  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+  frame = gimp_frame_new (_("Table Creation Options"));
   gtk_box_pack_start (GTK_BOX (main_vbox), frame, FALSE, FALSE, 0);
 
   table = gtk_table_new (4, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_container_add (GTK_CONTAINER (frame), table);
 
-  toggle = gtk_check_button_new_with_label (_("Use Cellspan"));
+  toggle = gtk_check_button_new_with_mnemonic (_("_Use cellspan"));
   gtk_table_attach (GTK_TABLE (table), toggle, 0, 2, 0, 1, GTK_FILL, 0, 0, 0);
-  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &gtmvals.spantags);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), gtmvals.spantags);
   gtk_widget_show (toggle);
+
   gimp_help_set_help_data (toggle,
 			   _("If checked GTM will replace any rectangular "
 			     "sections of identically colored blocks with one "
 			     "large cell with ROWSPAN and COLSPAN values."),
 			   NULL);
 
-  toggle = gtk_check_button_new_with_label (_("Compress TD tags"));
+  g_signal_connect (toggle, "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &gtmvals.spantags);
+
+  toggle = gtk_check_button_new_with_mnemonic (_("Co_mpress TD tags"));
   gtk_table_attach (GTK_TABLE (table), toggle, 0, 2, 1, 2, GTK_FILL, 0, 0, 0);
-  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &gtmvals.tdcomp);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), gtmvals.tdcomp);
   gtk_widget_show (toggle);
+
   gimp_help_set_help_data (toggle,
 			   _("Checking this tag will cause GTM to leave no "
 			     "whitespace between the TD tags and the "
@@ -481,152 +497,159 @@ save_dialog (image_ID)
 			     "level positioning control."),
 			   NULL);
 
-  toggle = gtk_check_button_new_with_label (_("Caption"));
+  g_signal_connect (toggle, "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &gtmvals.tdcomp);
+
+  toggle = gtk_check_button_new_with_mnemonic (_("C_aption"));
   gtk_table_attach (GTK_TABLE (table), toggle, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
-  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
-		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
-		      &gtmvals.caption);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), gtmvals.caption);
   gtk_widget_show (toggle);
+
   gimp_help_set_help_data (toggle,
 			   _("Check if you would like to have the table "
 			     "captioned."),
 			   NULL);
 
+  g_signal_connect (toggle, "toggled",
+                    G_CALLBACK (gimp_toggle_button_update),
+                    &gtmvals.caption);
+
   entry = gtk_entry_new ();
+  gtk_widget_set_size_request (entry, 200, -1);
+  gtk_entry_set_text (GTK_ENTRY (entry), gtmvals.captiontxt);
   gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 2, 3,
 		    GTK_FILL | GTK_EXPAND, 0, 0, 0);
-  gtk_widget_set_usize (entry, 200, 0);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-		      GTK_SIGNAL_FUNC (gtm_caption_callback),
-		      NULL);
-  gtk_entry_set_text (GTK_ENTRY (entry), gtmvals.captiontxt);
   gtk_widget_show (entry);
+
   gimp_help_set_help_data (entry, _("The text for the table caption."), NULL);
 
-  gtk_object_set_data (GTK_OBJECT (toggle), "set_sensitive", entry);
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (gtm_caption_callback),
+                    NULL);
+
+  g_object_set_data (G_OBJECT (toggle), "set_sensitive", entry);
   gtk_widget_set_sensitive (entry, gtmvals.caption);
 
   entry = gtk_entry_new ();
-  gtk_widget_set_usize (entry, 200, 0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 3,
-			     _("Cell Content:"), 1.0, 0.5,
-			     entry, 1, FALSE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-		      GTK_SIGNAL_FUNC (gtm_cellcontent_callback),
-		      NULL);
+  gtk_widget_set_size_request (entry, 200, -1);
   gtk_entry_set_text (GTK_ENTRY (entry), gtmvals.cellcontent);
+  gimp_table_attach_aligned (GTK_TABLE (table), 0, 3,
+			     _("C_ell content:"), 0.0, 0.5,
+			     entry, 1, FALSE);
   gtk_widget_show (entry);
+
   gimp_help_set_help_data (entry, _("The text to go into each cell."), NULL);
 
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (gtm_cellcontent_callback),
+                    NULL);
+
   gtk_widget_show (table);
   gtk_widget_show (frame);
 
   /* HTML Table Options */
-  frame = gtk_frame_new (_("Table Options"));
-  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+  frame = gimp_frame_new (_("Table Options"));
   gtk_box_pack_start (GTK_BOX (main_vbox), frame, FALSE, FALSE, 0);
 
   table = gtk_table_new (5, 2, FALSE);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 4);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
   gtk_container_add (GTK_CONTAINER (frame), table);
 
   spinbutton = gimp_spin_button_new (&adj, gtmvals.border,
 				     0, 1000, 1, 10, 0, 1, 0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-			     _("Border:"), 1.0, 0.5,
+			     _("_Border:"), 0.0, 0.5,
 			     spinbutton, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
-                      GTK_SIGNAL_FUNC (gimp_int_adjustment_update),
-                      &gtmvals.border);
+
   gimp_help_set_help_data (spinbutton,
 			   _("The number of pixels in the table border."),
 			   NULL);
 
+  g_signal_connect (adj, "value_changed",
+                    G_CALLBACK (gimp_int_adjustment_update),
+                    &gtmvals.border);
+
   entry = gtk_entry_new ();
-  gtk_widget_set_usize (entry, 60, 0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-			     _("Width:"), 1.0, 0.5,
-			     entry, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-                      GTK_SIGNAL_FUNC (gtm_clwidth_callback),
-                      NULL);
+  gtk_widget_set_size_request (entry, 60, -1);
   gtk_entry_set_text (GTK_ENTRY (entry), gtmvals.clwidth);
+  gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
+			     _("_Width:"), 0.0, 0.5,
+			     entry, 1, TRUE);
+
   gimp_help_set_help_data (entry,
 			   _("The width for each table cell.  "
 			     "Can be a number or a percent."),
 			   NULL);
 
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (gtm_clwidth_callback),
+                    NULL);
+
   entry = gtk_entry_new ();
-  gtk_widget_set_usize (entry, 60, 0);
-  gimp_table_attach_aligned (GTK_TABLE (table), 0, 2,
-			     _("Height:"), 1.0, 0.5,
-			     entry, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-                      GTK_SIGNAL_FUNC (gtm_clheight_callback),
-                      NULL);
+  gtk_widget_set_size_request (entry, 60, -1);
   gtk_entry_set_text (GTK_ENTRY (entry), gtmvals.clheight);
+  gimp_table_attach_aligned (GTK_TABLE (table), 0, 2,
+			     _("_Height:"), 0.0, 0.5,
+			     entry, 1, TRUE);
+
   gimp_help_set_help_data (entry,
 			   _("The height for each table cell.  "
 			     "Can be a number or a percent."),
 			   NULL);
 
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (gtm_clheight_callback),
+                    NULL);
+
   spinbutton = gimp_spin_button_new (&adj, gtmvals.cellpadding,
 				     0, 1000, 1, 10, 0, 1, 0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 3,
-			     _("Cell-Padding:"), 1.0, 0.5,
+			     _("Cell-_padding:"), 0.0, 0.5,
 			     spinbutton, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
-                      GTK_SIGNAL_FUNC (gimp_int_adjustment_update),
-                      &gtmvals.cellpadding);
+
   gimp_help_set_help_data (spinbutton,
 			   _("The amount of cellpadding."), NULL);
 
+  g_signal_connect (adj, "value_changed",
+                    G_CALLBACK (gimp_int_adjustment_update),
+                    &gtmvals.cellpadding);
+
   spinbutton = gimp_spin_button_new (&adj, gtmvals.cellspacing,
 				     0, 1000, 1, 10, 0, 1, 0);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 4,
-			     _("Cell-Spacing:"), 1.0, 0.5,
+			     _("Cell-_spacing:"), 0.0, 0.5,
 			     spinbutton, 1, TRUE);
-  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
-                      GTK_SIGNAL_FUNC (gimp_int_adjustment_update),
-                      &gtmvals.cellspacing);
+
   gimp_help_set_help_data (spinbutton,
 			   _("The amount of cellspacing."), NULL);
 
+  g_signal_connect (adj, "value_changed",
+                    G_CALLBACK (gimp_int_adjustment_update),
+                    &gtmvals.cellspacing);
+
   gtk_widget_show (table);
   gtk_widget_show (frame);
 
   gtk_widget_show (dlg);
 
-  gtk_main ();
-  gdk_flush ();
+  run = (gimp_dialog_run (GIMP_DIALOG (dlg)) == GTK_RESPONSE_OK);
 
-  return bint.run;
+  gtk_widget_destroy (dlg);
+
+  return run;
 }
 
-static gint
+static gboolean
 color_comp (guchar *buffer,
 	    guchar *buf2)
 {
-  if (buffer[0] == buf2[0] && buffer[1] == buf2[1] && buffer[2] == buf2[2])
-    return 1;
-  else
-    return 0;
-}  
+  return buffer[0] == buf2[0] && buffer[1] == buf2[1] && buffer[2] == buf2[2];
+}
 
 /*  Save interface functions  */
 
-static void
-save_ok_callback (GtkWidget *widget,
-		  gpointer   data)
-{
-  bint.run = TRUE;
-
-  gtk_widget_destroy (GTK_WIDGET (data));
-}
-
 static void
 gtm_caption_callback (GtkWidget *widget,
 		      gpointer   data)
