@@ -1,6 +1,6 @@
 /* The GIMP -- an image manipulation program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
- * Copyright (C) 1997 Daniel Risacher 
+ * Copyright (C) 1997 Daniel Risacher
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,75 +33,48 @@
  *      images are piped to uuencode and then to mail...
  *
  *
- *   This works fine for .99.10. I havent actually tried it in combination with
- *   the gz plugin, but it works with all other file types. I will eventually get
- *   around to making sure it works with gz.
+ *   This works fine for .99.10. I havent actually tried it in
+ *   combination with the gz plugin, but it works with all other file
+ *   types. I will eventually get around to making sure it works with
+ *   gz.
  *
  *  To use: 1) image->File->mail image
- *          2) when the mail dialog popups up, fill it out. Only to: and filename are required
- *             note: the filename needs to a type that the image can be saved as. otherwise
- *                   you will just send an empty message.
+ *          2) when the mail dialog popups up, fill it out. Only to:
+ *             and filename are required note: the filename needs to a
+ *             type that the image can be saved as. otherwise you will
+ *             just send an empty message.
  *          3) click ok and it should be on its way
  *
  *
- * NOTE: You probabaly need sendmail installed. If your sendmail is in an odd spot
- *       you can change the #define below. If you use qmail or other MTA's, and this
- *       works after changing the MAILER, let me know how well or what changes were
- *       needed.
+ * NOTE: You probabaly need sendmail installed. If your sendmail is in
+ *       an odd spot you can change the #define below. If you use
+ *       qmail or other MTA's, and this works after changing the
+ *       MAILER, let me know how well or what changes were needed.
  *
- * NOTE: Uuencode is needed. If it is in the path, it should work fine as is. Other-
- *       wise just change the UUENCODE.
+ * NOTE: Uuencode is needed. If it is in the path, it should work fine
+ *       as is. Other- wise just change the UUENCODE.
  *
  *
- * TODO: 1) the aforementioned abilty to specify the 
+ * TODO: 1) the aforementioned abilty to specify the
  *           uuencode filename                         *done*
  *       2) someway to do this without tmp files
  *              * wont happen anytime soon*
  *       3) MIME? *done*
  *       4) a pointlessly snazzier dialog
- *       5) make sure it works with gz     
+ *       5) make sure it works with gz
  *               * works for .xcfgz but not .xcf.gz *
- *       6) add an option to choose if mail get 
+ *       6) add an option to choose if mail get
  *          uuencode or not (or MIME'ed for that matter)
  *       7) realtime preview
  *       8) better entry for comments    *done*
- *       9) list of frequently used addreses     
+ *       9) list of frequently used addreses
  *      10) openGL compliance
  *      11) better handling of filesave errors
- *     
  *
- *  Version history
- *       .5  - 6/30/97 - inital relese
- *       .51 - 7/3/97  - fixed a few spelling errors and the like
- *       .65 - 7/4/97  - a fairly significant revision. changed it from a file
- *                       plugin to an image plugin.
- *                     - Changed some strcats into strcpy to be a bit more robust.
- *                     - added the abilty to specify the filename you want it sent as
- *                     - no more annoying hassles with the file saves as dialog
- *                     - plugin now registers itself as <image>/File/Mail image
- *       .7  - 9/12/97 - (RB) added support for MIME encapsulation
- *       .71 - 9/17/97 - (RB) included Base64 encoding functions from mpack
- *                       instead of using external program.
- *                     - General cleanup of the MIME handling code.
- *       .80 - 6/23/98 - Added a text box so you can compose real messages.
- *       .85 - 3/19/99 - Added a "From:" field. Made it check gimprc for a
- *                       "gump-from" token and use it. Also made "run with last 
- *                        values" work.
+ *
  * As always: The utility of this plugin is left as an exercise for the reader
  *
  */
-#ifndef MAILER
-#define MAILER "/usr/lib/sendmail"
-#endif
-
-#ifndef UUENCODE
-#define UUENCODE "uuencode"
-#endif
-
-#define ENCAPSULATION_UUENCODE 0
-#define ENCAPSULATION_MIME     1
-
-#define BUFFER_SIZE            256
 
 #include "config.h"
 
@@ -114,10 +87,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
-#ifdef __EMX__
-#include <fcntl.h>
-#include <process.h>
-#endif
 
 #include <gtk/gtk.h>
 
@@ -127,38 +96,102 @@
 #include "libgimp/stdplugins-intl.h"
 
 
+/* GdkPixbuf RGBA C-Source image dump 1-byte-run-length-encoded */
+
+#ifdef __SUNPRO_C
+#pragma align 4 (mail_icon)
+#endif
+#ifdef __GNUC__
+static const guint8 mail_icon[] __attribute__ ((__aligned__ (4))) =
+#else
+static const guint8 mail_icon[] =
+#endif
+{ ""
+  /* Pixbuf magic (0x47646b50) */
+  "GdkP"
+  /* length: header (24) + pixel_data (473) */
+  "\0\0\1\361"
+  /* pixdata_type (0x2010002) */
+  "\2\1\0\2"
+  /* rowstride (64) */
+  "\0\0\0@"
+  /* width (16) */
+  "\0\0\0\20"
+  /* height (16) */
+  "\0\0\0\20"
+  /* pixel_data: */
+  "\261\0\0\0\0\3\0\0\0e\0\0\0\377\40\40\40\377\211\0\0\0\377\1\0\0\0e\203"
+  "\0\0\0\0\2\0\0\0\377ttt\377\211\365\365\365\377\2DDD\377\0\0\0\377\203"
+  "\0\0\0\0\15\0\0\0\377\365\365\365\377ggg\377\361\361\361\377\343\343"
+  "\343\377\365\365\365\377\343\343\343\377\363\363\363\377\343\343\343"
+  "\377\330\330\330\377VVV\377\270\270\270\377\0\0\0\377\203\0\0\0\0\15"
+  "\0\0\0\377\365\365\365\377\343\343\343\377hhh\377\363\363\363\377\345"
+  "\345\345\377\362\362\362\377\343\343\343\377\326\326\326\377YYY\377\326"
+  "\326\326\377\260\260\260\377\0\0\0\377\203\0\0\0\0\2\0\0\0\377\365\365"
+  "\365\377\202\343\343\343\377\11eee\377\363\363\363\377\343\343\343\377"
+  "\333\333\333\377YYY\377\343\343\343\377\323\323\323\377\260\260\260\377"
+  "\0\0\0\377\203\0\0\0\0\15\0\0\0\377\365\365\365\377\343\343\343\377\336"
+  "\336\336\377\177\177\177\377YYY\377\365\365\365\377YYY\377nnn\377\330"
+  "\330\330\377\306\306\306\377\257\257\257\377\0\0\0\377\203\0\0\0\0\4"
+  "\0\0\0\377\365\365\365\377\340\340\340\377yyy\377\202\343\343\343\377"
+  "\1YYY\377\202\333\333\333\377\4eee\377\266\266\266\377\257\257\257\377"
+  "\0\0\0\377\203\0\0\0\0\15\0\0\0\377\365\365\365\377{{{\377\343\343\343"
+  "\377\323\323\323\377\334\334\334\377\325\325\325\377\333\333\333\377"
+  "\324\324\324\377\270\270\270\377JJJ\377\265\265\265\377\0\0\0\377\203"
+  "\0\0\0\0\3\0\0\0\377ZZZ\377\252\252\252\377\203\253\253\253\377\1\252"
+  "\252\252\377\202\254\254\254\377\4\255\255\255\377\262\262\262\377DD"
+  "D\377\0\0\0\377\203\0\0\0\0\1\0\0\0e\213\0\0\0\377\1\0\0\0e\262\0\0\0"
+  "\0"};
+
+
+
+#define ENCAPSULATION_UUENCODE 0
+#define ENCAPSULATION_MIME     1
+
+#define BUFFER_SIZE            256
+
+#ifndef SENDMAIL
+#define SENDMAIL "/usr/lib/sendmail"
+#endif
+
+#ifndef UUENCODE
+#define UUENCODE "uuencode"
+#endif
+
+#define PLUG_IN_NAME  "plug_in_mail_image"
+#define HELP_ID       "plug-in-mail-image"
+
+
 static void   query (void);
-static void   run   (gchar   *name,
-		     gint     nparams,
-		     GParam  *param,
-		     gint    *nreturn_vals,
-		     GParam **return_vals);
+static void   run   (const gchar      *name,
+		     gint              nparams,
+		     const GimpParam  *param,
+		     gint             *nreturn_vals,
+		     GimpParam       **return_vals);
 
-static GStatusType save_image (gchar  *filename,
-			       gint32  image_ID,
-			       gint32  drawable_ID,
-			       gint32  run_mode);
+static GimpPDBStatusType save_image (const gchar *filename,
+			             gint32       image_ID,
+			             gint32       drawable_ID,
+				     gint32       run_mode);
 
-static gint   save_dialog          (void);
-static void   ok_callback          (GtkWidget *widget,
-				    gpointer   data);
-static void   mail_entry_callback  (GtkWidget *widget,
-				    gpointer   data);
-static void   mesg_body_callback   (GtkWidget *widget,
-				    gpointer   data);
+static gint    save_dialog          (void);
+static void    mail_entry_callback  (GtkWidget     *widget,
+                                     gpointer       data);
+static void    mesg_body_callback   (GtkTextBuffer *buffer,
+                                     gpointer       data);
 
-static gint   valid_file     (gchar *filename);
-static void   create_headers (FILE  *mailpipe);
-static char * find_extension (gchar *filename);
-static gint   to64           (FILE  *infile,
-			      FILE  *outfile);
-static void   output64chunk  (gint   c1,
-			      gint   c2,
-			      gint   c3,
-			      gint   pads,
-			      FILE  *outfile);
+static gint    valid_file     (const gchar *filename);
+static void    create_headers (FILE        *mailpipe);
+static gchar * find_extension (const gchar *filename);
+static gint    to64           (FILE        *infile,
+                               FILE        *outfile);
+static void    output64chunk  (gint         c1,
+                               gint         c2,
+                               gint         c3,
+                               gint         pads,
+                               FILE        *outfile);
 
-GPlugInInfo PLUG_IN_INFO =
+GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
@@ -189,55 +222,59 @@ static m_info mail_info =
 			  if you prefer that as the default */
 };
 
-static gchar * mesg_body = "\0";
-static gint    run_flag  = 0;
+static gchar * mesg_body = NULL;
+
 
 MAIN ()
 
 static void
 query (void)
 {
-  static GParamDef args[] =
+  static GimpParamDef args[] =
   {
-    { PARAM_INT32, "run_mode", "Interactive, non-interactive" },
-    { PARAM_IMAGE, "image", "Input image" },
-    { PARAM_DRAWABLE, "drawable", "Drawable to save" },
-    { PARAM_STRING, "filename", "The name of the file to save the image in" },
-    { PARAM_STRING, "receipt", "The email address to send to" },
-    { PARAM_STRING, "from", "The email address for the From: field" },
-    { PARAM_STRING, "subject", "The subject" },
-    { PARAM_STRING, "comment", "The Comment" },
-    { PARAM_INT32,  "encapsulation", "Uuencode, MIME" }
+    { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
+    { GIMP_PDB_IMAGE, "image", "Input image" },
+    { GIMP_PDB_DRAWABLE, "drawable", "Drawable to save" },
+    { GIMP_PDB_STRING, "filename", "The name of the file to save the image in" },
+    { GIMP_PDB_STRING, "receipt", "The email address to send to" },
+    { GIMP_PDB_STRING, "from", "The email address for the From: field" },
+    { GIMP_PDB_STRING, "subject", "The subject" },
+    { GIMP_PDB_STRING, "comment", "The Comment" },
+    { GIMP_PDB_INT32,  "encapsulation", "Uuencode, MIME" }
   };
-  static gint nargs = sizeof (args) / sizeof (args[0]);
 
-  gimp_install_procedure ("plug_in_mail_image",
+  gimp_install_procedure (PLUG_IN_NAME,
 			  "pipe files to uuencode then mail them",
 			  "You need to have uuencode and mail installed",
 			  "Adrian Likins, Reagan Blundell",
-			  "Adrian Likins, Reagan Blundell, Daniel Risacher, Spencer Kimball and Peter Mattis",
+			  "Adrian Likins, Reagan Blundell, Daniel Risacher, "
+                          "Spencer Kimball and Peter Mattis",
 			  "1995-1997",
-			  N_("<Image>/File/Mail Image..."),
+			  N_("_Mail Image..."),
 			  "RGB*, GRAY*, INDEXED*",
-			  PROC_PLUG_IN,
-			  nargs, 0,
+			  GIMP_PLUGIN,
+			  G_N_ELEMENTS (args), 0,
 			  args, NULL);
+
+  gimp_plugin_menu_register (PLUG_IN_NAME, "<Image>/File/Send");
+  gimp_plugin_icon_register (PLUG_IN_NAME,
+                             GIMP_ICON_TYPE_INLINE_PIXBUF, mail_icon);
 }
 
 static void
-run (gchar   *name,
-     gint     nparams,
-     GParam  *param,
-     gint    *nreturn_vals,
-     GParam **return_vals)
+run (const gchar      *name,
+     gint              nparams,
+     const GimpParam  *param,
+     gint             *nreturn_vals,
+     GimpParam       **return_vals)
 {
-  static GParam values[2];
-  GRunModeType  run_mode;
-  GStatusType   status = STATUS_SUCCESS;
-  gint32        image_ID;
-  gint32        drawable_ID;
+  static GimpParam   values[2];
+  GimpRunMode        run_mode;
+  GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
+  gint32             image_ID;
+  gint32             drawable_ID;
 
-  INIT_I18N_UI();
+  INIT_I18N ();
 
   run_mode    = param[0].data.d_int32;
   image_ID    = param[1].data.d_image;
@@ -245,24 +282,25 @@ run (gchar   *name,
 
   *nreturn_vals = 1;
   *return_vals  = values;
-  values[0].type          = PARAM_STATUS;
-  values[0].data.d_status = STATUS_EXECUTION_ERROR;
 
-  if (strcmp (name, "plug_in_mail_image") == 0)
+  values[0].type          = GIMP_PDB_STATUS;
+  values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
+
+  if (strcmp (name, PLUG_IN_NAME) == 0)
     {
       switch (run_mode)
 	{
-	case RUN_INTERACTIVE:
-	  gimp_get_data ("plug_in_mail_image", &mail_info);
+	case GIMP_RUN_INTERACTIVE:
+	  gimp_get_data (PLUG_IN_NAME, &mail_info);
 	  if (!save_dialog ())
-	    status = STATUS_CANCEL;
+	    status = GIMP_PDB_CANCEL;
 	  break;
 
-	case RUN_NONINTERACTIVE:
+	case GIMP_RUN_NONINTERACTIVE:
 	  /*  Make sure all the arguments are there!  */
 	  if (nparams != 9)
 	    {
-	      status = STATUS_CALLING_ERROR;
+	      status = GIMP_PDB_CALLING_ERROR;
 	    }
 	  else
 	    {
@@ -276,122 +314,97 @@ run (gchar   *name,
 	    }
 	  break;
 
-	case RUN_WITH_LAST_VALS:
-	  gimp_get_data ("plug_in_mail_image", &mail_info);
+	case GIMP_RUN_WITH_LAST_VALS:
+	  gimp_get_data (PLUG_IN_NAME, &mail_info);
 	  break;
-	  
+
 	default:
 	  break;
 	}
 
-      if (status == STATUS_SUCCESS)
+      if (status == GIMP_PDB_SUCCESS)
 	{
 	  status = save_image (mail_info.filename,
 			       image_ID,
 			       drawable_ID,
 			       run_mode);
 
-	  if (status == STATUS_SUCCESS)
-	    gimp_set_data ("plug_in_mail_image", &mail_info, sizeof(m_info));
+	  if (status == GIMP_PDB_SUCCESS)
+	    gimp_set_data (PLUG_IN_NAME, &mail_info, sizeof(m_info));
 	}
     }
   else
     {
-      status = STATUS_CALLING_ERROR;
+      status = GIMP_PDB_CALLING_ERROR;
     }
 
   values[0].data.d_status = status;
 }
 
-static GStatusType
-save_image (gchar  *filename,
-	    gint32  image_ID,
-	    gint32  drawable_ID,
-	    gint32  run_mode)
+static GimpPDBStatusType
+save_image (const gchar *filename,
+	    gint32       image_ID,
+	    gint32       drawable_ID,
+	    gint32       run_mode)
 {
-  GParam *params;
-  gint    retvals;
   gchar  *ext;
   gchar  *tmpname;
   gchar   mailcmdline[512];
   gint    pid;
-  gint    status;
+  gint    wpid;
   gint    process_status;
   FILE   *mailpipe;
   FILE   *infile;
 
   if (NULL == (ext = find_extension (filename)))
-    return STATUS_CALLING_ERROR;
+    return GIMP_PDB_CALLING_ERROR;
 
   /* get a temp name with the right extension and save into it. */
-  params = gimp_run_procedure ("gimp_temp_name",
-			       &retvals,
-			       PARAM_STRING, ext + 1,
-			       PARAM_END);
-
-  tmpname = g_strdup (params[1].data.d_string);
-  gimp_destroy_params (params, retvals);
+  tmpname = gimp_temp_name (ext + 1);
 
   /* construct the "sendmail user@location" line */
-  strcpy (mailcmdline, MAILER);
+  strcpy (mailcmdline, SENDMAIL);
   strcat (mailcmdline, " ");
   strcat (mailcmdline, mail_info.receipt);
 
   /* create a pipe to sendmail */
-#ifndef __EMX__
   mailpipe = popen (mailcmdline, "w");
-#else
-  mailpipe = popen (mailcmdline, "wb");
-#endif
+
   create_headers (mailpipe);
-  
+
   /* This is necessary to make the comments and headers work correctly. Not real sure why */
-  fflush (mailpipe);      
+  fflush (mailpipe);
 
-  params = gimp_run_procedure ("gimp_file_save",
-			       &retvals,
-			       PARAM_INT32, run_mode,
-			       PARAM_IMAGE, image_ID,
-			       PARAM_DRAWABLE, drawable_ID,
-			       PARAM_STRING, tmpname,
-			       PARAM_STRING, tmpname,
-			       PARAM_END);
-  
-
-  /*  need to figure a way to make sure the user is trying to save
-   *  in an approriate format but this can wait....
-   */
-
-  status = params[0].data.d_status;
-
-  if (! valid_file (tmpname) ||
-      status != STATUS_SUCCESS)
+  if (! (gimp_file_save (run_mode,
+			 image_ID,
+			 drawable_ID,
+			 tmpname,
+			 tmpname) && valid_file (tmpname)) )
     {
       unlink (tmpname);
       g_free (tmpname);
-      return status;
+      return GIMP_PDB_EXECUTION_ERROR;
     }
 
   if (mail_info.encapsulation == ENCAPSULATION_UUENCODE)
     {
-#ifndef __EMX__
       /* fork off a uuencode process */
       if ((pid = fork ()) < 0)
 	{
-	  g_message ("mail: fork failed: %s\n", g_strerror (errno));
+	  g_message ("fork() failed: %s", g_strerror (errno));
 	  g_free (tmpname);
-	  return STATUS_EXECUTION_ERROR;
+	  return GIMP_PDB_EXECUTION_ERROR;
 	}
       else if (pid == 0)
 	{
 	  if (-1 == dup2 (fileno (mailpipe), fileno (stdout)))
 	    {
-	      g_message ("mail: dup2 failed: %s\n", g_strerror (errno));
+	      g_message ("dup2() failed: %s", g_strerror (errno));
 	    }
 
 	  execlp (UUENCODE, UUENCODE, tmpname, filename, NULL);
 	  /* What are we doing here? exec must have failed */
-	  g_message ("mail: exec failed: uuencode: %s\n", g_strerror (errno));
+	  g_message ("exec failed: uuencode: %s", g_strerror (errno));
 
 	  /* close the pipe now */
 	  pclose (mailpipe);
@@ -399,38 +412,17 @@ save_image (gchar  *filename,
 	  _exit (127);
 	}
       else
-#else /* __EMX__ */
-      int tfd;
-      /* save fileno(stdout) */
-      tfd = dup (fileno (stdout));
-      if (dup2 (fileno (mailpipe), fileno (stdout)) == -1)
-	{
-	  g_message ("mail: dup2 failed: %s\n", g_strerror (errno));
-	  close (tfd);
-	  g_free (tmpname);
-	  return STATUS_EXECUTION_ERROR;
-	}
-      fcntl (tfd, F_SETFD, FD_CLOEXEC);
-      pid = spawnlp (P_NOWAIT, UUENCODE, UUENCODE, tmpname, filename, NULL);
-      /* restore fileno(stdout) */
-      dup2 (tfd, fileno (stdout));
-      close (tfd);
-      if (pid == -1)
-	{
-	  g_message ("mail: spawn failed: %s\n", g_strerror (errno));
-	  g_free (tmpname);
-	  return STATUS_EXECUTION_ERROR;
-	}
-#endif
         {
-	  waitpid (pid, &process_status, 0);
+	  wpid = waitpid (pid, &process_status, 0);
 
-	  if (!WIFEXITED (process_status) ||
-	      WEXITSTATUS (process_status) != 0)
+	  if ((wpid < 0)
+	      || !WIFEXITED (process_status)
+	      || (WEXITSTATUS (process_status) != 0))
 	    {
-	      g_message ("mail: mail didnt work or something on file %s\n", tmpname);
+	      g_message ("mail didnt work or something on file '%s'",
+                         gimp_filename_to_utf8 (tmpname));
 	      g_free (tmpname);
-	      return STATUS_EXECUTION_ERROR;
+	      return GIMP_PDB_EXECUTION_ERROR;
 	    }
 	}
     }
@@ -449,29 +441,29 @@ save_image (gchar  *filename,
   unlink (tmpname);
   g_free (tmpname);
 
-  return STATUS_SUCCESS;
+  return GIMP_PDB_SUCCESS;
 }
 
 
 static gint
 save_dialog (void)
 {
-  GtkWidget *dlg;
-  GtkWidget *entry;
-  GtkWidget *table;
-  GtkWidget *table2;
-  GtkWidget *label;
-  GtkWidget *vbox;
-  GtkWidget *text;
-  GtkWidget *vscrollbar;
-
-  gchar      buffer[BUFFER_SIZE];
-  gchar     *gump_from;
+  GtkWidget     *dlg;
+  GtkWidget     *entry;
+  GtkWidget     *table;
+  GtkWidget     *label;
+  GtkWidget     *vbox;
+  GtkWidget     *scrolled_window;
+  GtkWidget     *text_view;
+  GtkTextBuffer *text_buffer;
+  gchar          buffer[BUFFER_SIZE];
+  gchar         *gump_from;
+  gboolean       run;
 
   gimp_ui_init ("mail", FALSE);
 
-  /* check gimprc for a preffered "From:" address */
-  gump_from = gimp_gimprc_query ("gump-from");         
+  /* check gimprc for a preferred "From:" address */
+  gump_from = gimp_gimprc_query ("gump-from");
 
   if (gump_from)
     {
@@ -479,154 +471,143 @@ save_dialog (void)
       g_free (gump_from);
     }
 
-  dlg = gimp_dialog_new (_("Send to Mail"), "mail",
-			 gimp_standard_help_func, "filters/mail.html",
-			 GTK_WIN_POS_MOUSE,
-			 FALSE, TRUE, FALSE,
+  dlg = gimp_dialog_new (_("Send as Mail"), "mail",
+                         NULL, 0,
+			 gimp_standard_help_func, HELP_ID,
 
-			 _("OK"), ok_callback,
-			 NULL, NULL, NULL, TRUE, FALSE,
-			 _("Cancel"), gtk_widget_destroy,
-			 NULL, 1, NULL, FALSE, TRUE,
+			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
 			 NULL);
 
-  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
-		      GTK_SIGNAL_FUNC (gtk_main_quit),
-		      NULL);
-
   /* table */
   table = gtk_table_new (7, 2, FALSE);
-  gtk_container_border_width (GTK_CONTAINER (table), 6);
+  gtk_container_set_border_width (GTK_CONTAINER (table), 12);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), table, TRUE, TRUE, 0);
   gtk_widget_show (table);
 
-  gtk_table_set_row_spacings (GTK_TABLE (table), 4);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 
   /* to: dialog */
   entry = gtk_entry_new ();
-  gtk_widget_set_usize (entry, 200, 0);
+  gtk_widget_set_size_request (entry, 200, -1);
   g_snprintf (buffer, sizeof (buffer), "%s", mail_info.receipt);
   gtk_entry_set_text (GTK_ENTRY (entry), buffer);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-			     _("Recipient:"), 1.0, 0.5,
+			     _("_Recipient:"), 0.0, 0.5,
 			     entry, 1, FALSE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-		      GTK_SIGNAL_FUNC (mail_entry_callback),
-		      &mail_info.receipt);
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (mail_entry_callback),
+                    &mail_info.receipt);
 
   /* From entry */
   entry = gtk_entry_new ();
-  gtk_widget_set_usize (entry, 200, 0);
+  gtk_widget_set_size_request (entry, 200, -1);
   g_snprintf (buffer, sizeof (buffer), "%s", mail_info.from);
   gtk_entry_set_text (GTK_ENTRY (entry), buffer);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
-			     _("Sender:"), 1.0, 0.5,
+			     _("_Sender:"), 0.0, 0.5,
 			     entry, 1, FALSE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-		      GTK_SIGNAL_FUNC (mail_entry_callback),
-		      &mail_info.from);
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (mail_entry_callback),
+                    &mail_info.from);
 
   /* Subject entry */
   entry = gtk_entry_new ();
-  gtk_widget_set_usize (entry, 200, 0);
+  gtk_widget_set_size_request (entry, 200, -1);
   g_snprintf (buffer, sizeof (buffer), "%s", mail_info.subject);
   gtk_entry_set_text (GTK_ENTRY (entry), buffer);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 2,
-			     _("Subject:"), 1.0, 0.5,
+			     _("S_ubject:"), 0.0, 0.5,
 			     entry, 1, FALSE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-		      GTK_SIGNAL_FUNC (mail_entry_callback),
-		      &mail_info.subject);
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (mail_entry_callback),
+                    &mail_info.subject);
 
   /* Comment entry */
   entry = gtk_entry_new ();
-  gtk_widget_set_usize (entry, 200, 0);
+  gtk_widget_set_size_request (entry, 200, -1);
   g_snprintf (buffer, sizeof (buffer), "%s", mail_info.comment);
   gtk_entry_set_text (GTK_ENTRY (entry), buffer);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 3,
-			     _("Comment:"), 1.0, 0.5,
+			     _("Comm_ent:"), 0.0, 0.5,
 			     entry, 1, FALSE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-		      GTK_SIGNAL_FUNC (mail_entry_callback),
-		      &mail_info.comment);
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (mail_entry_callback),
+                    &mail_info.comment);
 
   /* Filename entry */
   entry = gtk_entry_new ();
-  gtk_widget_set_usize (entry, 200, 0);
+  gtk_widget_set_size_request (entry, 200, -1);
   g_snprintf (buffer, sizeof (buffer), "%s", mail_info.filename);
   gtk_entry_set_text (GTK_ENTRY (entry), buffer);
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 4,
-			     _("Filename:"), 1.0, 0.5,
+			     _("_Filename:"), 0.0, 0.5,
 			     entry, 1, FALSE);
-  gtk_signal_connect (GTK_OBJECT (entry), "changed",
-		      GTK_SIGNAL_FUNC (mail_entry_callback),
-		      &mail_info.filename);
+  g_signal_connect (entry, "changed",
+                    G_CALLBACK (mail_entry_callback),
+                    &mail_info.filename);
 
   /* comment  */
-  table2 = gtk_table_new (2, 2, FALSE);
-  gtk_table_set_row_spacing (GTK_TABLE (table2), 0, 2);
-  gtk_table_set_col_spacing (GTK_TABLE (table2), 0, 2);
+  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
+                                       GTK_SHADOW_IN);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+                                  GTK_POLICY_AUTOMATIC,
+                                  GTK_POLICY_AUTOMATIC);
+  gtk_table_attach (GTK_TABLE (table), scrolled_window,
+                    0, 2, 5, 6,
+                    GTK_EXPAND | GTK_FILL,
+                    GTK_EXPAND | GTK_FILL,
+                    0, 0);
+  gtk_widget_show (scrolled_window);
 
-  gtk_table_attach (GTK_TABLE (table), table2, 
-		    0, 2, 5, 6,
-		    GTK_EXPAND | GTK_FILL,
-		    GTK_EXPAND | GTK_FILL,
-		    0, 0);
+  text_buffer = gtk_text_buffer_new (NULL);
 
-  text = gtk_text_new (NULL, NULL);
-  gtk_text_set_editable (GTK_TEXT (text), TRUE);
-  gtk_table_attach (GTK_TABLE (table2), text, 
-		    0, 1, 0, 1,
-		    GTK_EXPAND | GTK_FILL,
-		    GTK_EXPAND | GTK_FILL,
-		    0, 0);
-  gtk_widget_set_usize (text, 200, 100);
-  gtk_widget_show (text);
-  gtk_signal_connect (GTK_OBJECT (text), "changed",
-		      GTK_SIGNAL_FUNC (mesg_body_callback),
-		      mesg_body);
-  gtk_widget_show (table2);
+  text_view = gtk_text_view_new_with_buffer (text_buffer);
+  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD);
+  gtk_container_add (GTK_CONTAINER (scrolled_window), text_view);
+  gtk_widget_show (text_view);
 
-  vscrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
-  gtk_table_attach (GTK_TABLE (table2), vscrollbar, 1, 2, 0, 1,
-		    GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
-  gtk_widget_show (vscrollbar);
+  g_object_unref (text_buffer);
+
+  g_signal_connect (text_buffer, "changed",
+                    G_CALLBACK (mesg_body_callback),
+		    NULL);
 
   /* Encapsulation label */
   label = gtk_label_new (_("Encapsulation:"));
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.2);
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 6, 7,
 		    GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
   gtk_widget_show (label);
 
   /* Encapsulation radiobuttons */
-  vbox = gimp_radio_group_new2 (FALSE, NULL,
-				gimp_radio_button_update,
-				&mail_info.encapsulation,
-				(gpointer) mail_info.encapsulation,
+  vbox = gimp_int_radio_group_new (FALSE, NULL,
+				   G_CALLBACK (gimp_radio_button_update),
+				   &mail_info.encapsulation,
+				   mail_info.encapsulation,
 
-				_("Uuencode"),
-				(gpointer) ENCAPSULATION_UUENCODE, NULL,
-				_("MIME"),
-				(gpointer) ENCAPSULATION_MIME, NULL,
+				   _("_Uuencode"), ENCAPSULATION_UUENCODE, NULL,
+				   _("_MIME"),     ENCAPSULATION_MIME,     NULL,
 
-				NULL);
+				   NULL);
   gtk_table_attach (GTK_TABLE (table), vbox, 1, 2, 6, 8,
 		    GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
   gtk_widget_show (vbox);
 
   gtk_widget_show (dlg);
 
-  gtk_main ();
-  gdk_flush ();
+  run = (gimp_dialog_run (GIMP_DIALOG (dlg)) == GTK_RESPONSE_OK);
 
-  return run_flag;
+  gtk_widget_destroy (dlg);
+
+  return run;
 }
 
-static gint 
-valid_file (gchar *filename)
+static gint
+valid_file (const gchar *filename)
 {
   int stat_res;
   struct stat buf;
@@ -640,11 +621,11 @@ valid_file (gchar *filename)
 }
 
 static gchar *
-find_content_type (gchar *filename)
+find_content_type (const gchar *filename)
 {
   /* This function returns a MIME Content-type: value based on the
      filename it is given.  */
-  gchar *type_mappings[20] =
+  const gchar *type_mappings[20] =
   {
     "gif" , "image/gif",
     "jpg" , "image/jpeg",
@@ -659,7 +640,7 @@ find_content_type (gchar *filename)
   };
 
   gchar *ext;
-  gchar *mimetype = malloc(100);
+  gchar *mimetype = g_new (gchar, 100);
   gint i=0;
 
   ext = find_extension (filename);
@@ -686,7 +667,7 @@ find_content_type (gchar *filename)
 }
 
 static gchar *
-find_extension (gchar *filename)
+find_extension (const gchar *filename)
 {
   gchar *filename_copy;
   gchar *ext;
@@ -709,12 +690,12 @@ find_extension (gchar *filename)
     {
       if (!ext || ext[1] == 0 || strchr (ext, '/'))
 	{
-	  g_message (_("mail: some sort of error with the file extension or lack thereof \n"));
-	  
+	  g_message (_("some sort of error with the file extension or lack thereof"));
+
 	  return NULL;
 	}
       if (0 != strcmp(ext,".gz"))
-	{ 
+	{
 	  return ext;
 	}
       else
@@ -727,15 +708,6 @@ find_extension (gchar *filename)
   return ext;
 }
 
-static void
-ok_callback (GtkWidget *widget,
-	     gpointer   data)
-{
-  run_flag = TRUE;
-  
-  gtk_widget_destroy (GTK_WIDGET (data));
-}
-
 static void
 mail_entry_callback (GtkWidget *widget,
 		     gpointer   data)
@@ -743,13 +715,20 @@ mail_entry_callback (GtkWidget *widget,
   strncpy ((gchar *) data, gtk_entry_get_text (GTK_ENTRY (widget)), BUFFER_SIZE);
 }
 
-static void 
-mesg_body_callback (GtkWidget *widget,
-		    gpointer   data)
+static void
+mesg_body_callback (GtkTextBuffer *buffer,
+		    gpointer       data)
 {
-  mesg_body = gtk_editable_get_chars (GTK_EDITABLE (widget), 0,
-				      gtk_text_get_length (GTK_TEXT (widget)));
-} 
+  GtkTextIter start_iter;
+  GtkTextIter end_iter;
+
+
+  if (mesg_body)
+    g_free (mesg_body);
+
+  gtk_text_buffer_get_bounds (buffer, &start_iter, &end_iter);
+  mesg_body = gtk_text_buffer_get_text (buffer, &start_iter, &end_iter, FALSE);
+}
 
 static void
 create_headers (FILE *mailpipe)
@@ -777,41 +756,32 @@ create_headers (FILE *mailpipe)
     }
   fprintf (mailpipe, mail_info.comment);
   fprintf (mailpipe, "\n\n");
-  fprintf (mailpipe, mesg_body); 
+  if (mesg_body)
+    fprintf (mailpipe, mesg_body);
   fprintf (mailpipe, "\n\n");
   if (mail_info.encapsulation == ENCAPSULATION_MIME )
     {
-      char *content;
-      content = find_content_type (mail_info.filename);
+      gchar *content = find_content_type (mail_info.filename);
+
       fprintf (mailpipe, "--GUMP-MIME-boundary\n");
       fprintf (mailpipe, "Content-type: %s\n",content);
       fprintf (mailpipe, "Content-transfer-encoding: base64\n");
-      fprintf (mailpipe, "Content-disposition: attachment; filename=\"%s\"\n",mail_info.filename);
+      fprintf (mailpipe, "Content-disposition: attachment; filename=\"%s\"\n",
+               mail_info.filename);
       fprintf (mailpipe, "Content-description: %s\n\n",mail_info.filename);
-      free(content);
+
+      g_free (content);
     }
 }
 
 /*
  * The following code taken from codes.c in the mpack-1.5 distribution
- * by Carnegie Mellon University. 
- * 
+ * by Carnegie Mellon University.
+ *
  *
  * (C) Copyright 1993,1994 by Carnegie Mellon University
  * All Rights Reserved.
  *
- * Permission to use, copy, modify, distribute, and sell this software
- * and its documentation for any purpose is hereby granted without
- * fee, provided that the above copyright notice appear in all copies
- * and that both that copyright notice and this permission notice
- * appear in supporting documentation, and that the name of Carnegie
- * Mellon University not be used in advertising or publicity
- * pertaining to distribution of the software without specific,
- * written prior permission.  Carnegie Mellon University makes no
- * representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied
- * warranty.
- *
  * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
  * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
@@ -824,15 +794,15 @@ create_headers (FILE *mailpipe)
 /*
 Copyright (c) 1991 Bell Communications Research, Inc. (Bellcore)
 
-Permission to use, copy, modify, and distribute this material 
-for any purpose and without fee is hereby granted, provided 
-that the above copyright notice and this permission notice 
-appear in all copies, and that the name of Bellcore not be 
-used in advertising or publicity pertaining to this 
-material without the specific, prior written permission 
-of an authorized representative of Bellcore.  BELLCORE 
-MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY 
-OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", 
+Permission to use, copy, modify, and distribute this material
+for any purpose and without fee is hereby granted, provided
+that the above copyright notice and this permission notice
+appear in all copies, and that the name of Bellcore not be
+used in advertising or publicity pertaining to this
+material without the specific, prior written permission
+of an authorized representative of Bellcore.  BELLCORE
+MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY
+OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS",
 WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.  */
 
 
@@ -841,7 +811,7 @@ static gchar basis_64[] =
 
 static gint
 to64 (FILE *infile,
-      FILE *outfile) 
+      FILE *outfile)
 {
   gint c1, c2, c3, ct = 0, written = 0;
 
