@@ -20,7 +20,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- * 
+ *
  */
 
 /* Change log:-
@@ -29,18 +29,15 @@
 
 #include "config.h"
 
-#include <glib.h>
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <math.h>
 
-#include <gtk/gtk.h>
-
-#include <libgimp/gimp.h>
-#include <libgimp/gimpui.h>
+#include "libgimp/gimp.h"
+#include "libgimp/gimpui.h"
 
+#include "libgimpmath/gimpmath.h"
+#
 #include "global.h"
 #include "types.h"
 #include "pxl-outline.h"
@@ -51,27 +48,26 @@
 #include "libgimp/stdplugins-intl.h"
 
 
-#define MID_POINT 127
+#define RESPONSE_RESET 1
+#define MID_POINT      127
 
 /***** Magic numbers *****/
 
 /* Variables set in dialog box */
 
 static void      query  (void);
-static void      run    (gchar    *name,
-			 gint      nparams,
-			 GimpParam   *param,
-			 gint     *nreturn_vals,
-			 GimpParam  **return_vals);
+static void      run    (const gchar      *name,
+			 gint              nparams,
+			 const GimpParam  *param,
+			 gint             *nreturn_vals,
+			 GimpParam       **return_vals);
 
 static gint      sel2path_dialog         (SELVALS   *sels);
-static void      sel2path_ok_callback    (GtkWidget *widget, 
-					  gpointer   data);
-static void      sel2path_reset_callback (GtkWidget *widget,
+static void      sel2path_response       (GtkWidget *widget,
+                                          gint       response_id,
 					  gpointer   data);
 static void      dialog_print_selVals    (SELVALS   *sels);
-gboolean         do_sel2path             (gint32     drawable_ID, 
-					  gint32     image_ID);
+gboolean         do_sel2path             (gint32     image_ID);
 
 
 GimpPlugInInfo PLUG_IN_INFO =
@@ -85,19 +81,26 @@ GimpPlugInInfo PLUG_IN_INFO =
 static gint    sel_x1, sel_y1, sel_x2, sel_y2;
 static gint    has_sel, sel_width, sel_height;
 static SELVALS selVals;
-GimpPixelRgn      selection_rgn;
+GimpPixelRgn   selection_rgn;
 gboolean       retVal = TRUE;  /* Toggle if cancle button clicked */
 
 MAIN ()
 
 static void
-query_2 (void)
+query (void)
 {
   static GimpParamDef args[] =
+  {
+    { GIMP_PDB_INT32,    "run_mode", "Interactive, non-interactive" },
+    { GIMP_PDB_IMAGE,    "image",    "Input image" },
+    { GIMP_PDB_DRAWABLE, "drawable", "Input drawable (unused)" },
+  };
+
+  static GimpParamDef advanced_args[] =
   {
     { GIMP_PDB_INT32,    "run_mode",                    "Interactive, non-interactive" },
-    { GIMP_PDB_IMAGE,    "image",                       "Input image (unused)" },
-    { GIMP_PDB_DRAWABLE, "drawable",                    "Input drawable" }, 
+    { GIMP_PDB_IMAGE,    "image",                       "Input image" },
+    { GIMP_PDB_DRAWABLE, "drawable",                    "Input drawable (unused)" },
     { GIMP_PDB_FLOAT,    "align_threshold",             "align_threshold"},
     { GIMP_PDB_FLOAT,    "corner_always_threshold",     "corner_always_threshold"},
     { GIMP_PDB_INT8,     "corner_surround",             "corner_surround"},
@@ -119,9 +122,18 @@ query_2 (void)
     { GIMP_PDB_FLOAT,    "subdivide_threshold",         "subdivide_threshold"},
     { GIMP_PDB_INT8,     "tangent_surround",            "tangent_surround"},
   };
-  static GimpParamDef *return_vals = NULL;
-  static int nargs = sizeof (args) / sizeof (args[0]);
-  static int nreturn_vals = 0;
+
+  gimp_install_procedure ("plug_in_sel2path",
+			  "Converts a selection to a path",
+			  "Converts a selection to a path",
+			  "Andy Thomas",
+			  "Andy Thomas",
+			  "1999",
+			  NULL,
+			  "RGB*, INDEXED*, GRAY*",
+			  GIMP_PLUGIN,
+			  G_N_ELEMENTS (args), 0,
+			  args, NULL);
 
   gimp_install_procedure ("plug_in_sel2path_advanced",
 			  "Converts a selection to a path (with advanced user menu)",
@@ -132,64 +144,28 @@ query_2 (void)
 			  NULL,
 			  "RGB*, INDEXED*, GRAY*",
 			  GIMP_PLUGIN,
-			  nargs, nreturn_vals,
-			  args, return_vals);
+			  G_N_ELEMENTS (advanced_args), 0,
+			  advanced_args, NULL);
 }
 
 static void
-query (void)
+run (const gchar      *name,
+     gint              nparams,
+     const GimpParam  *param,
+     gint             *nreturn_vals,
+     GimpParam       **return_vals)
 {
-  static GimpParamDef args[] =
-  {
-    { GIMP_PDB_INT32,    "run_mode", "Interactive, non-interactive" },
-    { GIMP_PDB_IMAGE,    "image",    "Input image (unused)" },
-    { GIMP_PDB_DRAWABLE, "drawable", "Input drawable" },
-  };
-  static GimpParamDef *return_vals = NULL;
-  static int nargs = sizeof (args) / sizeof (args[0]);
-  static int nreturn_vals = 0;
-
-  INIT_I18N();
-
-  gimp_install_procedure ("plug_in_sel2path",
-			  "Converts a selection to a path",
-			  "Converts a selection to a path",
-			  "Andy Thomas",
-			  "Andy Thomas",
-			  "1999",
-			  N_("<Image>/Select/To Path"),
-			  "RGB*, INDEXED*, GRAY*",
-			  GIMP_PLUGIN,
-			  nargs, nreturn_vals,
-			  args, return_vals);
-
-  query_2 ();
-}
-
-static void
-run (gchar    *name,
-     gint      nparams,
-     GimpParam   *param,
-     gint     *nreturn_vals,
-     GimpParam  **return_vals)
-{
-  static GimpParam values[1];
-  GimpDrawable *   drawable;
-  gint32        drawable_ID;
-  gint32        image_ID;
-  GimpRunModeType  run_mode;
-  GimpPDBStatusType   status = GIMP_PDB_SUCCESS;
-  gboolean      no_dialog = FALSE;
+  static GimpParam   values[1];
+  gint32             image_ID;
+  GimpRunMode        run_mode;
+  GimpPDBStatusType  status    = GIMP_PDB_SUCCESS;
+  gboolean           no_dialog;
 
   run_mode = param[0].data.d_int32;
 
-  if(!strcmp(name,"plug_in_sel2path")) 
-    {
-      no_dialog = TRUE;
-      INIT_I18N();
-    } 
-  else
-    INIT_I18N_UI();
+  INIT_I18N ();
+
+  no_dialog = (strcmp (name, "plug_in_sel2path") == 0);
 
   *nreturn_vals = 1;
   *return_vals = values;
@@ -197,21 +173,22 @@ run (gchar    *name,
   values[0].type = GIMP_PDB_STATUS;
   values[0].data.d_status = status;
 
-  drawable_ID = param[2].data.d_drawable;
-  drawable = gimp_drawable_get (drawable_ID);
-
-  image_ID = gimp_drawable_image_id(drawable_ID);
-
-  if(gimp_selection_is_empty(image_ID))
+  image_ID = param[1].data.d_image;
+  if (image_ID < 0)
     {
-      g_message(_("No selection to convert"));
-      gimp_drawable_detach (drawable);
+      g_warning ("plug_in_sel2path needs a valid image ID");
       return;
     }
 
-  fit_set_default_params(&selVals);
-  
-  if(!no_dialog)
+  if (gimp_selection_is_empty (image_ID))
+    {
+      g_message (_("No selection to convert"));
+      return;
+    }
+
+  fit_set_default_params (&selVals);
+
+  if (!no_dialog)
     {
       switch (run_mode)
 	{
@@ -222,19 +199,17 @@ run (gchar    *name,
 	    }
 
 	  if (!sel2path_dialog (&selVals))
-	    {
-	      gimp_drawable_detach (drawable);
-	      return;
-	    }
+            return;
+
 	  /* Get the current settings */
 	  fit_set_params (&selVals);
 	  break;
-	  
+
 	case GIMP_RUN_NONINTERACTIVE:
 	  if (nparams != 23)
 	    status = GIMP_PDB_CALLING_ERROR;
-	  
-	  if (status == GIMP_PDB_SUCCESS) 
+
+	  if (status == GIMP_PDB_SUCCESS)
 	    {
 	      selVals.align_threshold             =  param[3].data.d_float;
 	      selVals.corner_always_threshold     =  param[4].data.d_float;
@@ -256,28 +231,26 @@ run (gchar    *name,
 	      selVals.subdivide_surround          = param[20].data.d_int8;
 	      selVals.subdivide_threshold         = param[21].data.d_float;
 	      selVals.tangent_surround            = param[22].data.d_int8;
-	      fit_set_params(&selVals);
-	  }
-
+	      fit_set_params (&selVals);
+            }
 	  break;
-	  
+
 	case GIMP_RUN_WITH_LAST_VALS:
 	  if(gimp_get_data_size ("plug_in_sel2path_advanced") > 0)
 	    {
 	      gimp_get_data ("plug_in_sel2path_advanced", &selVals);
-	      
+
 	      /* Set up the last values */
 	      fit_set_params (&selVals);
 	    }
-
 	  break;
-	  
+
 	default:
 	  break;
 	}
     }
 
-  do_sel2path (drawable_ID,image_ID);
+  do_sel2path (image_ID);
   values[0].data.d_status = status;
 
   if (status == GIMP_PDB_SUCCESS)
@@ -286,8 +259,6 @@ run (gchar    *name,
       if (run_mode == GIMP_RUN_INTERACTIVE && !no_dialog)
 	gimp_set_data ("plug_in_sel2path_advanced", &selVals, sizeof(SELVALS));
     }
-
-  gimp_drawable_detach (drawable);
 }
 
 static void
@@ -328,28 +299,25 @@ sel2path_dialog (SELVALS *sels)
 
   gimp_ui_init ("sel2path", FALSE);
 
-  dlg = gimp_dialog_new (_("Sel2Path Advanced Settings"), "sel2path",
-			 gimp_standard_help_func, "filters/sel2path.html",
-			 GTK_WIN_POS_MOUSE,
-			 FALSE, TRUE, FALSE,
+  dlg = gimp_dialog_new (_("Selection To Path Advanced Settings"), "sel2path",
+                         NULL, 0,
+			 gimp_standard_help_func, "plug-in-sel2path-advanced",
 
-			 _("OK"), sel2path_ok_callback,
-			 NULL, NULL, NULL, TRUE, FALSE,
-
-			 _("Reset"), sel2path_reset_callback,
-			 NULL, NULL, NULL, FALSE, FALSE,
-
-			 _("Cancel"), gtk_widget_destroy,
-			 NULL, 1, NULL, FALSE, TRUE,
+			 GIMP_STOCK_RESET, RESPONSE_RESET,
+			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
 			 NULL);
 
-  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
-		      GTK_SIGNAL_FUNC (gtk_main_quit),
-		      NULL);
-  
-  table = dialog_create_selection_area(sels);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 6);
+  g_signal_connect (dlg, "response",
+                    G_CALLBACK (sel2path_response),
+                    NULL);
+  g_signal_connect (dlg, "destroy",
+                    G_CALLBACK (gtk_main_quit),
+                    NULL);
+
+  table = dialog_create_selection_area (sels);
+  gtk_container_set_border_width (GTK_CONTAINER (table), 12);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dlg)->vbox), table);
   gtk_widget_show (table);
 
@@ -361,29 +329,33 @@ sel2path_dialog (SELVALS *sels)
 }
 
 static void
-sel2path_ok_callback (GtkWidget *widget,
-		      gpointer   data)
+sel2path_response (GtkWidget *widget,
+                   gint       response_id,
+                   gpointer   data)
 {
-  gtk_widget_destroy (GTK_WIDGET (data));
-  retVal = TRUE;
-}
+  switch (response_id)
+    {
+    case RESPONSE_RESET:
+      reset_adv_dialog ();
+      fit_set_params (&selVals);
+      break;
 
-static void
-sel2path_reset_callback (GtkWidget *widget,
-			 gpointer   data)
-{
-  reset_adv_dialog();
-  fit_set_params(&selVals);
+    case GTK_RESPONSE_OK:
+      retVal = TRUE;
+
+    default:
+      gtk_widget_destroy (widget);
+      break;
+    }
 }
 
 guchar
-sel_pixel_value (gint row, 
+sel_pixel_value (gint row,
 		 gint col)
 {
   guchar ret;
 
-  if(col > sel_width ||
-     row > sel_height)
+  if (col > sel_width || row > sel_height)
     {
       g_warning ("sel_pixel_value [%d,%d] out of bounds", col, row);
       return 0;
@@ -395,7 +367,7 @@ sel_pixel_value (gint row,
 }
 
 gboolean
-sel_pixel_is_white (gint row, 
+sel_pixel_is_white (gint row,
 		    gint col)
 {
   if (sel_pixel_value (row, col) < MID_POINT)
@@ -404,7 +376,7 @@ sel_pixel_is_white (gint row,
     return FALSE;
 }
 
-gint 
+gint
 sel_get_width (void)
 {
   return sel_width;
@@ -416,8 +388,8 @@ sel_get_height (void)
   return sel_height;
 }
 
-gint 
-sel_valid_pixel (gint row, 
+gint
+sel_valid_pixel (gint row,
 		 gint col)
 {
   return (0 <= (row) && (row) < sel_get_height ()
@@ -439,9 +411,9 @@ gen_anchor (gdouble  *p,
   *p++ = sel_y1 + sel_height - (gint)RINT(y) + 1;
   *p++ = (is_newcurve) ? 3.0 : 1.0;
 }
-  
 
-void  
+
+void
 gen_control (gdouble *p,
 	     gdouble  x,
 	     gdouble  y)
@@ -495,25 +467,25 @@ do_points (spline_list_array_type in_splines,
     {
       unsigned this_spline;
       spline_list_type in_list = SPLINE_LIST_ARRAY_ELT (in_splines, this_list);
-      
+
 /*       if(seg_count > 0 && point_count > 0)   */
 /* 	gen_anchor(last_x,last_y,0);   */
-      
+
       point_count = 0;
 
       /* Ignore single points that are on their own */
       if(SPLINE_LIST_LENGTH (in_list) < 2)
 	  continue;
-      
+
       for (this_spline = 0; this_spline < SPLINE_LIST_LENGTH (in_list);
 	   this_spline++)
 	{
 	  spline_type s = SPLINE_LIST_ELT (in_list, this_spline);
-	  
+
 	  if (SPLINE_DEGREE (s) == LINEAR)
 	    {
-	      gen_anchor (cur_parray, 
-			  START_POINT (s).x, START_POINT (s).y, 
+	      gen_anchor (cur_parray,
+			  START_POINT (s).x, START_POINT (s).y,
 			  seg_count && !point_count);
 	      cur_parray += 3;
 	      gen_control (cur_parray, START_POINT (s).x, START_POINT (s).y);
@@ -537,32 +509,30 @@ do_points (spline_list_array_type in_splines,
 	      last_y = END_POINT (s).y;
 	    }
 	  else
-	    g_message ( _("print_spline: strange degree (%d)"), 
-			SPLINE_DEGREE (s));
-	  
+	    g_warning ("print_spline: strange degree (%d)", SPLINE_DEGREE (s));
+
 	  point_count++;
 	}
       seg_count++;
     }
 
    gimp_path_set_points (image_ID,
- 			_("selection_to_path"), 
- 			1, 
- 			path_point_count,
- 			parray);
+ 			"selection_to_path",
+                         1,
+                         path_point_count,
+                         parray);
 }
 
 
 gboolean
-do_sel2path (gint32 drawable_ID,
-	     gint32 image_ID)
+do_sel2path (gint32 image_ID)
 {
-  gint32      selection_ID;
-  GimpDrawable  *sel_drawable;
-  pixel_outline_list_type olt;
-  spline_list_array_type splines;
+  gint32                   selection_ID;
+  GimpDrawable            *sel_drawable;
+  pixel_outline_list_type  olt;
+  spline_list_array_type   splines;
 
-  gimp_selection_bounds (image_ID, &has_sel, 
+  gimp_selection_bounds (image_ID, &has_sel,
 			 &sel_x1, &sel_y1, &sel_x2, &sel_y2);
 
   sel_width  = sel_x2 - sel_x1;
@@ -570,19 +540,16 @@ do_sel2path (gint32 drawable_ID,
 
   /* Now get the selection channel */
 
-  selection_ID = gimp_image_get_selection(image_ID);
+  selection_ID = gimp_image_get_selection (image_ID);
 
-  if(selection_ID < 0)
-    {
-      g_message( _("gimp_image_get_selection failed"));
-      return FALSE;
-    }
+  if (selection_ID < 0)
+    return FALSE;
 
   sel_drawable = gimp_drawable_get (selection_ID);
 
-  if(gimp_drawable_bpp(selection_ID) != 1)
+  if (gimp_drawable_bpp (selection_ID) != 1)
     {
-      g_message( _("Internal error. Selection bpp > 1"));
+      g_warning ("Internal error. Selection bpp > 1");
       return FALSE;
     }
 
@@ -590,30 +557,27 @@ do_sel2path (gint32 drawable_ID,
 		       sel_x1, sel_y1, sel_width, sel_height,
 		       FALSE, FALSE);
 
-  gimp_tile_cache_ntiles (2 * (sel_drawable->width + gimp_tile_width () - 1) / 
+  gimp_tile_cache_ntiles (2 * (sel_drawable->width + gimp_tile_width () - 1) /
 			  gimp_tile_width ());
 
   olt = find_outline_pixels ();
 
   splines = fitted_splines (olt);
 
-  gimp_selection_none (image_ID);
-  gimp_displays_flush ();
-
   do_points (splines, image_ID);
 
+  gimp_drawable_detach (sel_drawable);
+
+  gimp_displays_flush ();
+
   return TRUE;
 }
 
 void
 safe_free (address *item)
 {
-  if (item == NULL || *item == NULL)
-    {
-      g_warning ("safe_free: Attempt to free a null item.");
-    }
-  
+  g_return_if_fail (item != NULL);
+
   g_free (*item);
-  
   *item = NULL;
 }
